<?php
$tmp = pow(2,24);
$data = array(
    'PHP',
    17=>'PHP: Hypertext Preprocessor',
    5=>'Test',
    'test'=>27,
    1000=>'test',
    "-1000"=>array('banana', 'orange'),
    'monkey',
    $tmp=>-1/3
);
?>
