/* Generated by re2c 3.1 */

enum YYCONDTYPE {
	yycINITIAL,
	yycST_OFFSET,
	yycST_SECTION_VALUE,
	yycST_VALUE,
	yycST_SECTION_RAW,
	yycST_DOUBLE_QUOTES,
	yycST_VAR_FALLBACK,
	yycST_VARNAME,
	yycST_RAW
};
